@@ -4,20 +4,20 @@
  @author Johan Pascal
 
  @copyright Copyright (C) 2014 Belledonne Communications, Grenoble, France
- 
+
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
- 
+
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #ifndef TYPEDEF_H
 #define TYPEDEF_H
@@ -48,11 +48,11 @@
 
 typedef struct bzrtpChannelContext_struct bzrtpChannelContext_t;
 
-#include "cryptoWrapper.h"
+#include <bctoolbox/crypto.h>
 #include "packetParser.h"
 #include "stateMachine.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 #define snprintf _snprintf
 #endif
 
@@ -117,7 +117,7 @@ typedef struct cachedSecretsHash_struct {
  *
  */
 struct bzrtpChannelContext_struct {
-	
+
 	void *clientData; /**< this is a pointer provided by the client which is then resent as a parameter of the callbacks functions. Usefull to store RTP session context for example */
 
 	uint8_t role;/**< can be INITIATOR or RESPONDER, is set to INITIATOR at creation, may switch to responder later */
@@ -128,6 +128,7 @@ struct bzrtpChannelContext_struct {
 
 	/* flags */
 	uint8_t isSecure; /**< This flag is set to 1 when the ZRTP negociation ends and SRTP secrets are generated and confirmed for this channel */
+	uint8_t isMainChannel; /**< this flag is set for the firt channel only, allow to distinguish channel to be secured using DHM or multiStream */
 
 	/* Hash chains, self is generated at channel context init */
 	uint8_t selfH[4][32]; /**< Store self 256 bits Hash images H0-H3 used to generate messages MAC */
@@ -137,6 +138,9 @@ struct bzrtpChannelContext_struct {
 	bzrtpPacket_t *selfPackets[PACKET_STORAGE_CAPACITY]; /**< Hello, Commit and DHPart packet locally generated */
 	bzrtpPacket_t *peerPackets[PACKET_STORAGE_CAPACITY]; /**< Hello, Commit and DHPart packet received from peer */
 
+	/* peer Hello hash : store the peer hello hash when given by signaling */
+	uint8_t *peerHelloHash; /**< peer hello hash - SHA256 of peer Hello packet, given through signaling, shall be a 32 bytes buffer */
+
 	/* sequence number: self and peer */
 	uint16_t selfSequenceNumber; /**< Sequence number of the next packet to be sent */
 	uint16_t peerSequenceNumber; /**< Sequence number of the last valid received packet */
@@ -179,16 +183,18 @@ struct bzrtpChannelContext_struct {
 };
 
 /**
- * @brief structure of the ZRTP engine context 
+ * @brief structure of the ZRTP engine context
  * Store current state, timers, HMAC and encryption keys
 */
 struct bzrtpContext_struct {
+	void *ZIDCacheData; /**<  this is a pointer provided by the client which is then resent as a parameter of the ZID cache related callbacks functions. */
 	/* contexts */
-	bzrtpRNGContext_t *RNGContext; /**< context for random number generation */
-	bzrtpDHMContext_t *DHMContext; /**< context for the Diffie-Hellman-Merkle operations. Only one DHM computation may be done during a call, so this belongs to the general context and not the channel one */
+	bctbx_rng_context_t *RNGContext; /**< context for random number generation */
+	bctbx_DHMContext_t *DHMContext; /**< context for the Diffie-Hellman-Merkle operations. Only one DHM computation may be done during a call, so this belongs to the general context and not the channel one */
 
 	/* flags */
-	uint8_t isSecure; /**< this flag is set to 1 after the first channel have completed the ZRTP protocol exchange(i.e. when the responder have sent the conf2ACK message) */
+	uint8_t isInitialised; /**< this flag is set once the context was initialised : self ZID retrieved from cache or generated, used to unlock the creation of addtional channels */
+	uint8_t isSecure; /**< this flag is set to 1 after the first channel have completed the ZRTP protocol exchange(i.e. when the responder have sent the conf2ACK message), must be set in order to start an additional channel */
 	uint8_t peerSupportMultiChannel; /**< this flag is set to 1 when the first valid HELLO packet from peer arrives if it support Multichannel ZRTP */
 
 	uint64_t timeReference; /**< in ms. This field will set at each channel State Machine start and updated at each tick after creation of the context, it is used to set the firing time of a channel timer */
@@ -210,7 +216,7 @@ struct bzrtpContext_struct {
 	uint8_t supportedKeyAgreement[7]; /**< list of supported key agreement algorithms mapped to uint8_t */
 	uint8_t sc; /**< sas count - set to 0 means we support only base32 (4 bits) */
 	uint8_t supportedSas[7]; /**< list of supported Sas representations mapped to uint8_t */
-	
+
 	/* ZIDs and cache */
 #ifdef HAVE_LIBXML2
 	xmlDocPtr cacheBuffer; /**< cache file is load in this buffer to be parsed/written */
