@@ -19,14 +19,14 @@
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <stdlib.h>
 #include <string.h>
 #include "typedef.h"
 #include "packetParser.h"
-#include "cryptoWrapper.h"
+#include <bctoolbox/crypto.h>
 #include "cryptoUtils.h"
 
 /* DEBUG */
@@ -196,9 +196,6 @@ bzrtpPacket_t *bzrtp_packetCheck(const uint8_t * input, uint16_t inputLength, ui
 }
 
 
-#define MIN(a, b) (((a) < (b)) ? (a) : (b))
-
-
 /* Call this function after the packetCheck one, to actually parse the packet : create and fill the messageData structure */
 int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpChannelContext, const uint8_t * input, uint16_t inputLength, bzrtpPacket_t *zrtpPacket) {
 
@@ -211,8 +208,24 @@ int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpC
 	switch (zrtpPacket->messageType) {
 		case MSGTYPE_HELLO : 
 			{
-				/* allocate a Hello message structure */
 				bzrtpHelloMessage_t *messageData;
+
+				/* Do we have a peerHelloHash to check */
+				if (zrtpChannelContext->peerHelloHash != NULL) {
+					uint8_t computedPeerHelloHash[32];
+					/* compute hash using implicit hash function: SHA256, skip packet header in the packetString buffer as the hash must be computed on message only */
+					bctbx_sha256(input+ZRTP_PACKET_HEADER_LENGTH,
+						inputLength - ZRTP_PACKET_OVERHEAD,
+						32,
+						computedPeerHelloHash);
+
+					/* check they are the same */
+					if (memcmp(computedPeerHelloHash, zrtpChannelContext->peerHelloHash, 32)!=0) {
+						return BZRTP_ERROR_HELLOHASH_MISMATCH;
+					}
+				}
+
+				/* allocate a Hello message structure */
 				messageData = (bzrtpHelloMessage_t *)malloc(sizeof(bzrtpHelloMessage_t));
 
 				/* fill it */
@@ -316,14 +329,14 @@ int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpC
 				}
 				peerHelloMessageData = (bzrtpHelloMessage_t *)zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageData;
 				/* Check H3 = SHA256(H2) */
-				bzrtpCrypto_sha256(messageData->H2, 32, 32, checkH3);
+				bctbx_sha256(messageData->H2, 32, 32, checkH3);
 				if (memcmp(checkH3, peerHelloMessageData->H3, 32) != 0) {
 					free (messageData);
 					return BZRTP_PARSER_ERROR_UNMATCHINGHASHCHAIN;
 				}
 				/* Check the hello MAC message. 
 				 * MAC is 8 bytes long and is computed on the message(skip the ZRTP_PACKET_HEADER) and exclude the mac itself (-8 bytes from message Length) */
-				bzrtpCrypto_hmacSha256(messageData->H2, 32, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
+				bctbx_hmacSha256(messageData->H2, 32, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
 				if (memcmp(checkMAC, peerHelloMessageData->MAC, 8) != 0) {
 					free (messageData);
 					return BZRTP_PARSER_ERROR_UNMATCHINGMAC;
@@ -426,18 +439,44 @@ int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpC
 					}
 					peerCommitMessageData = (bzrtpCommitMessage_t *)zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->messageData;
 					/* Check H2 = SHA256(H1) */
-					bzrtpCrypto_sha256(messageData->H1, 32, 32, checkH2);
+					bctbx_sha256(messageData->H1, 32, 32, checkH2);
 					if (memcmp(checkH2, peerCommitMessageData->H2, 32) != 0) {
 						free (messageData);
 						return BZRTP_PARSER_ERROR_UNMATCHINGHASHCHAIN;
 					}
 					/* Check the Commit MAC message. 
 					 * MAC is 8 bytes long and is computed on the message(skip the ZRTP_PACKET_HEADER) and exclude the mac itself (-8 bytes from message Length) */
-					bzrtpCrypto_hmacSha256(messageData->H1, 32, zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
+					bctbx_hmacSha256(messageData->H1, 32, zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
 					if (memcmp(checkMAC, peerCommitMessageData->MAC, 8) != 0) {
 						free (messageData);
 						return BZRTP_PARSER_ERROR_UNMATCHINGMAC;
 					}
+
+					/* Check the hvi received in the commit message  - RFC section 4.4.1.1*/
+					/* First compute the expected hvi */
+					/* hvi = hash(initiator's DHPart2 message(current zrtpPacket)) || responder's Hello message) using the agreed hash function truncated to 256 bits */
+					/* create a string with the messages concatenated */
+					{
+						uint8_t computedHvi[32];
+						uint16_t HelloMessageLength = zrtpChannelContext->selfPackets[HELLO_MESSAGE_STORE_ID]->messageLength;
+						uint16_t DHPartHelloMessageStringLength = zrtpPacket->messageLength + HelloMessageLength;
+
+						uint8_t *DHPartHelloMessageString = (uint8_t *)malloc(DHPartHelloMessageStringLength*sizeof(uint8_t));
+
+						memcpy(DHPartHelloMessageString, input+ZRTP_PACKET_HEADER_LENGTH, zrtpPacket->messageLength);
+						memcpy(DHPartHelloMessageString+zrtpPacket->messageLength, zrtpChannelContext->selfPackets[HELLO_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, HelloMessageLength);
+
+						zrtpChannelContext->hashFunction(DHPartHelloMessageString, DHPartHelloMessageStringLength, 32, computedHvi);
+
+						free(DHPartHelloMessageString);
+
+						/* Compare computed and received hvi */
+						if (memcmp(computedHvi, peerCommitMessageData->hvi, 32)!=0) {
+							free (messageData);
+							return BZRTP_PARSER_ERROR_UNMATCHINGHVI;
+						}
+					}
+
 				} else { /* if we are initiator(we didn't received any commit message and then no H2), we must check that H3=SHA256(SHA256(H1)) and the Hello message MAC */
 					uint8_t checkH2[32];
 					uint8_t checkH3[32];
@@ -451,15 +490,15 @@ int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpC
 					}
 					peerHelloMessageData = (bzrtpHelloMessage_t *)zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageData;
 					/* Check H3 = SHA256(SHA256(H1)) */
-					bzrtpCrypto_sha256(messageData->H1, 32, 32, checkH2);
-					bzrtpCrypto_sha256(checkH2, 32, 32, checkH3);
+					bctbx_sha256(messageData->H1, 32, 32, checkH2);
+					bctbx_sha256(checkH2, 32, 32, checkH3);
 					if (memcmp(checkH3, peerHelloMessageData->H3, 32) != 0) {
 						free (messageData);
 						return BZRTP_PARSER_ERROR_UNMATCHINGHASHCHAIN;
 					}
 					/* Check the hello MAC message. 
 					 * MAC is 8 bytes long and is computed on the message(skip the ZRTP_PACKET_HEADER) and exclude the mac itself (-8 bytes from message Length) */
-					bzrtpCrypto_hmacSha256(checkH2, 32, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
+					bctbx_hmacSha256(checkH2, 32, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
 					if (memcmp(checkMAC, peerHelloMessageData->MAC, 8) != 0) {
 						free (messageData);
 						return BZRTP_PARSER_ERROR_UNMATCHINGMAC;
@@ -482,7 +521,7 @@ int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpC
 				/* attach the message structure to the packet one */
 				zrtpPacket->messageData = (void *)messageData;
 
-				/* the parsed commit packet must be saved as it is used to generate the total_hash */
+				/* the parsed packet must be saved as it is used to generate the total_hash */
 				zrtpPacket->packetString = (uint8_t *)malloc(inputLength*sizeof(uint8_t));
 				memcpy(zrtpPacket->packetString, input, inputLength); /* store the whole packet even if we may use the message only */
 			}
@@ -550,7 +589,7 @@ int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpC
 				if (zrtpChannelContext->keyAgreementAlgo == ZRTP_KEYAGREEMENT_Prsh || zrtpChannelContext->keyAgreementAlgo == ZRTP_KEYAGREEMENT_Mult) {
 					/* compute the H1=SHA256(H0) we never received */
 					uint8_t checkH1[32];
-					bzrtpCrypto_sha256(messageData->H0, 32, 32, checkH1);
+					bctbx_sha256(messageData->H0, 32, 32, checkH1);
 
 					/* if we are responder, we received a commit packet with H2 then check that H2=SHA256(H1) and that the commit message MAC keyed with H1 match */
 					if ( zrtpChannelContext->role == RESPONDER) {
@@ -565,14 +604,14 @@ int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpC
 						}
 						peerCommitMessageData = (bzrtpCommitMessage_t *)zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->messageData;
 						/* Check H2 = SHA256(H1) */
-						bzrtpCrypto_sha256(checkH1, 32, 32, checkH2);
+						bctbx_sha256(checkH1, 32, 32, checkH2);
 						if (memcmp(checkH2, peerCommitMessageData->H2, 32) != 0) {
 							free (messageData);
 							return BZRTP_PARSER_ERROR_UNMATCHINGHASHCHAIN;
 						}
 						/* Check the Commit MAC message. 
 						 * MAC is 8 bytes long and is computed on the message(skip the ZRTP_PACKET_HEADER) and exclude the mac itself (-8 bytes from message Length) */
-						bzrtpCrypto_hmacSha256(checkH1, 32, zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
+						bctbx_hmacSha256(checkH1, 32, zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[COMMIT_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
 						if (memcmp(checkMAC, peerCommitMessageData->MAC, 8) != 0) {
 							free (messageData);
 							return BZRTP_PARSER_ERROR_UNMATCHINGMAC;
@@ -590,15 +629,15 @@ int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpC
 						}
 						peerHelloMessageData = (bzrtpHelloMessage_t *)zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageData;
 						/* Check H3 = SHA256(SHA256(H1)) */
-						bzrtpCrypto_sha256(checkH1, 32, 32, checkH2);
-						bzrtpCrypto_sha256(checkH2, 32, 32, checkH3);
+						bctbx_sha256(checkH1, 32, 32, checkH2);
+						bctbx_sha256(checkH2, 32, 32, checkH3);
 						if (memcmp(checkH3, peerHelloMessageData->H3, 32) != 0) {
 							free (messageData);
 							return BZRTP_PARSER_ERROR_UNMATCHINGHASHCHAIN;
 						}
 						/* Check the hello MAC message. 
 						 * MAC is 8 bytes long and is computed on the message(skip the ZRTP_PACKET_HEADER) and exclude the mac itself (-8 bytes from message Length) */
-						bzrtpCrypto_hmacSha256(checkH2, 32, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
+						bctbx_hmacSha256(checkH2, 32, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
 						if (memcmp(checkMAC, peerHelloMessageData->MAC, 8) != 0) {
 							free (messageData);
 							return BZRTP_PARSER_ERROR_UNMATCHINGMAC;
@@ -618,14 +657,14 @@ int bzrtp_packetParser(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpC
 					}
 					peerDHPartMessageData = (bzrtpDHPartMessage_t *)zrtpChannelContext->peerPackets[DHPART_MESSAGE_STORE_ID]->messageData;
 					/* Check H1 = SHA256(H0) */
-					bzrtpCrypto_sha256(messageData->H0, 32, 32, checkH1);
+					bctbx_sha256(messageData->H0, 32, 32, checkH1);
 					if (memcmp(checkH1, peerDHPartMessageData->H1, 32) != 0) {
 						free (messageData);
 						return BZRTP_PARSER_ERROR_UNMATCHINGHASHCHAIN;
 					}
 					/* Check the DHPart message. 
 					 * MAC is 8 bytes long and is computed on the message(skip the ZRTP_PACKET_HEADER) and exclude the mac itself (-8 bytes from message Length) */
-					bzrtpCrypto_hmacSha256(messageData->H0, 32, zrtpChannelContext->peerPackets[DHPART_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[DHPART_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
+					bctbx_hmacSha256(messageData->H0, 32, zrtpChannelContext->peerPackets[DHPART_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpChannelContext->peerPackets[DHPART_MESSAGE_STORE_ID]->messageLength-8, 8, checkMAC);
 					if (memcmp(checkMAC, peerDHPartMessageData->MAC, 8) != 0) {
 						free (messageData);
 						return BZRTP_PARSER_ERROR_UNMATCHINGMAC;
@@ -1041,7 +1080,7 @@ int bzrtp_packetBuild(bzrtpContext_t *zrtpContext, bzrtpChannelContext_t *zrtpCh
 		if (MACbuffer != NULL) {
 			/* compute the MAC(64 bits only) using the implicit HMAC function for ZRTP v1.10: HMAC-SHA256 */
 			/* HMAC is computed on the whole message except the MAC itself so a length of zrtpPacket->messageLength-8 */
-			bzrtpCrypto_hmacSha256(MACkey, 32, zrtpPacket->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpPacket->messageLength-8, 8, MACbuffer);
+			bctbx_hmacSha256(MACkey, 32, zrtpPacket->packetString+ZRTP_PACKET_HEADER_LENGTH, zrtpPacket->messageLength-8, 8, MACbuffer);
 		}
 	
 		/* set packet header and CRC */
@@ -1153,7 +1192,7 @@ bzrtpPacket_t *bzrtp_createZrtpPacket(bzrtpContext_t *zrtpContext, bzrtpChannelC
 
 				/* if it is a multistream or preshared commit create a 16 random bytes nonce */
 				if ((zrtpCommitMessage->keyAgreementAlgo == ZRTP_KEYAGREEMENT_Prsh) || (zrtpCommitMessage->keyAgreementAlgo == ZRTP_KEYAGREEMENT_Mult)) {
-					bzrtpCrypto_getRandom(zrtpContext->RNGContext, zrtpCommitMessage->nonce, 16);
+					bctbx_rng_get(zrtpContext->RNGContext, zrtpCommitMessage->nonce, 16);
 
 					/* and the keyID for preshared commit only */
 					if (zrtpCommitMessage->keyAgreementAlgo == ZRTP_KEYAGREEMENT_Prsh) {
@@ -1189,6 +1228,7 @@ bzrtpPacket_t *bzrtp_createZrtpPacket(bzrtpContext_t *zrtpContext, bzrtpChannelC
 		case MSGTYPE_DHPART2 :
 			{
 				uint8_t secretLength; /* is in bytes */
+				uint8_t bctbx_keyAgreementAlgo = BCTBX_DHM_UNSET;
 				bzrtpDHPartMessage_t *zrtpDHPartMessage = (bzrtpDHPartMessage_t *)malloc(sizeof(bzrtpDHPartMessage_t));
 				memset(zrtpDHPartMessage, 0, sizeof(bzrtpDHPartMessage_t));
 				/* initialise some fields using zrtp context data */
@@ -1217,7 +1257,22 @@ bzrtpPacket_t *bzrtp_createZrtpPacket(bzrtpContext_t *zrtpContext, bzrtpChannelC
 						secretLength = 32;
 						break;
 				}
-				zrtpContext->DHMContext = bzrtpCrypto_CreateDHMContext(zrtpChannelContext->keyAgreementAlgo, secretLength);
+
+				switch (zrtpChannelContext->keyAgreementAlgo) {
+					case ZRTP_KEYAGREEMENT_DH2k:
+						bctbx_keyAgreementAlgo = BCTBX_DHM_2048;
+						break;
+					case ZRTP_KEYAGREEMENT_DH3k:
+						bctbx_keyAgreementAlgo = BCTBX_DHM_3072;
+						break;
+					default:
+						free(zrtpPacket);
+						free(zrtpDHPartMessage);
+						*exitCode = BZRTP_CREATE_ERROR_UNABLETOCREATECRYPTOCONTEXT;
+						return NULL;
+						break;
+				}
+				zrtpContext->DHMContext = bctbx_CreateDHMContext(bctbx_keyAgreementAlgo, secretLength);
 				if (zrtpContext->DHMContext == NULL) {
 					free(zrtpPacket);
 					free(zrtpDHPartMessage);
@@ -1226,7 +1281,7 @@ bzrtpPacket_t *bzrtp_createZrtpPacket(bzrtpContext_t *zrtpContext, bzrtpChannelC
 				}
 
 				/* now compute the public value */
-				bzrtpCrypto_DHMCreatePublic(zrtpContext->DHMContext, (int (*)(void *, uint8_t *, size_t))bzrtpCrypto_getRandom, zrtpContext->RNGContext);
+				bctbx_DHMCreatePublic(zrtpContext->DHMContext, (int (*)(void *, uint8_t *, size_t))bctbx_rng_get, zrtpContext->RNGContext);
 				zrtpDHPartMessage->pv = (uint8_t *)malloc((zrtpChannelContext->keyAgreementLength)*sizeof(uint8_t));
 				memcpy(zrtpDHPartMessage->pv, zrtpContext->DHMContext->self, zrtpChannelContext->keyAgreementLength);
 
@@ -1250,7 +1305,7 @@ bzrtpPacket_t *bzrtp_createZrtpPacket(bzrtpContext_t *zrtpContext, bzrtpChannelC
 				zrtpConfirmMessage->D = 0; /* The is no backdoor in our implementation of ZRTP - rfc section 11 */
 
 				/* generate a random CFB IV */
-				bzrtpCrypto_getRandom(zrtpContext->RNGContext, zrtpConfirmMessage->CFBIV, 16);
+				bctbx_rng_get(zrtpContext->RNGContext, zrtpConfirmMessage->CFBIV, 16);
 
 				/* attach the message data to the packet */
 				zrtpPacket->messageData = zrtpConfirmMessage;
