@@ -22,13 +22,14 @@
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #include <stdlib.h>
 #include <string.h>
-#include "zidCache.h"
-
 #include "typedef.h"
+#include <bctoolbox/crypto.h>
+#include "cryptoUtils.h"
+#include "zidCache.h"
 
 #ifdef HAVE_LIBXML2
 
@@ -39,11 +40,7 @@
 #define XML_HEADER_STRING "<?xml version='1.0' encoding='utf-8'?>"
 #define XML_HEADER_SIZE 38
 /* Local functions prototypes */
-void bzrtp_strToUint8(uint8_t *outputBytes, uint8_t *inputString, uint16_t inputLength);
-void bzrtp_int8ToStr(uint8_t *outputString, uint8_t *inputBytes, uint16_t inputBytesLength);
-uint8_t bzrtp_byteToChar(uint8_t inputByte);
-uint8_t bzrtp_charToByte(uint8_t inputChar);
-void bzrtp_writeCache(bzrtpContext_t *zrtpContext);
+static void bzrtp_writeCache(bzrtpContext_t *zrtpContext);
 
 int bzrtp_getSelfZID(bzrtpContext_t *context, uint8_t selfZID[12]) {
 	uint8_t *selfZidHex = NULL;
@@ -56,12 +53,12 @@ int bzrtp_getSelfZID(bzrtpContext_t *context, uint8_t selfZID[12]) {
 		uint8_t *cacheStringBuffer;
 		uint32_t cacheStringLength;
 		zrtpFreeBuffer_callback cb=NULL;
-		context->zrtpCallbacks.bzrtp_loadCache(context->channelContext[0]->clientData, &cacheStringBuffer, &cacheStringLength, &cb);
+		context->zrtpCallbacks.bzrtp_loadCache(context->ZIDCacheData, &cacheStringBuffer, &cacheStringLength, &cb);
 		context->cacheBuffer = xmlParseDoc(cacheStringBuffer);
 		if (cb!=NULL) cb(cacheStringBuffer);
 	} else {
 		/* we are running cacheless, return a random number */
-		bzrtpCrypto_getRandom(context->RNGContext, selfZID, 12);
+		bctbx_rng_get(context->RNGContext, selfZID, 12);
 		return 0; 
 	}
 
@@ -90,7 +87,7 @@ int bzrtp_getSelfZID(bzrtpContext_t *context, uint8_t selfZID[12]) {
 		xmlNodePtr rootNode;
 
 		/* generate a random ZID */
-		bzrtpCrypto_getRandom(context->RNGContext, selfZID, 12);
+		bctbx_rng_get(context->RNGContext, selfZID, 12);
 		/* convert it to an Hexa String */
 		bzrtp_int8ToStr(newZidHex, selfZID, 12);
 		newZidHex[24] = '\0'; /* the string must be null terminated for libxml2 to add it correctly in the element */
@@ -251,7 +248,7 @@ int bzrtp_writePeerNode(bzrtpContext_t *context, uint8_t peerZID[12], uint8_t *t
 		/* reload cache from file locking it (TODO: lock) */
 		xmlFreeDoc(context->cacheBuffer);
 		context->cacheBuffer = NULL;
-		context->zrtpCallbacks.bzrtp_loadCache(context->channelContext[0]->clientData, &cacheStringBuffer, &cacheStringLength,&cb);
+		context->zrtpCallbacks.bzrtp_loadCache(context->ZIDCacheData, &cacheStringBuffer, &cacheStringLength,&cb);
 		context->cacheBuffer = xmlParseDoc(cacheStringBuffer);
 		if (cb) cb(cacheStringBuffer);
 	}
@@ -339,89 +336,15 @@ int bzrtp_writePeerNode(bzrtpContext_t *context, uint8_t peerZID[12], uint8_t *t
  * @param[in/out]	zrtpContext		The zrtp context containing the cacheBuffer
  *
  */
-void bzrtp_writeCache(bzrtpContext_t *zrtpContext) {
+static void bzrtp_writeCache(bzrtpContext_t *zrtpContext) {
 	/* dump the xml document into a string */
 	xmlChar *xmlStringOutput;
 	int xmlStringLength;
 	xmlDocDumpFormatMemoryEnc(zrtpContext->cacheBuffer, &xmlStringOutput, &xmlStringLength, "UTF-8", 0);
 	/* write it to the file */
-	zrtpContext->zrtpCallbacks.bzrtp_writeCache(zrtpContext->channelContext[0]->clientData, xmlStringOutput, xmlStringLength);
+	zrtpContext->zrtpCallbacks.bzrtp_writeCache(zrtpContext->ZIDCacheData, xmlStringOutput, xmlStringLength);
 	xmlFree(xmlStringOutput);
 }
-/**
- * @brief Convert an hexadecimal string into the corresponding byte buffer
- *
- * @param[out]	outputBytes			The output bytes buffer, must have a length of half the input string buffer
- * @param[in]	inputString			The input string buffer, must be hexadecimal(it is not checked by function, any non hexa char is converted to 0)
- * @param[in]	inputStringLength	The length in chars of the string buffer, output is half this length
- */
-void bzrtp_strToUint8(uint8_t *outputBytes, uint8_t *inputString, uint16_t inputStringLength) {
-	int i;
-	for (i=0; i<inputStringLength/2; i++) {
-		outputBytes[i] = (bzrtp_charToByte(inputString[2*i]))<<4 | bzrtp_charToByte(inputString[2*i+1]);
-	}
-}
-
-/**
- * @brief Convert a byte buffer into the corresponding hexadecimal string
- *
- * @param[out]	outputString		The output string buffer, must have a length of twice the input bytes buffer
- * @param[in]	inputBytes			The input bytes buffer
- * @param[in]	inputBytesLength	The length in bytes buffer, output is twice this length
- */
-void bzrtp_int8ToStr(uint8_t *outputString, uint8_t *inputBytes, uint16_t inputBytesLength) {
-	int i;
-	for (i=0; i<inputBytesLength; i++) {
-		outputString[2*i] = bzrtp_byteToChar((inputBytes[i]>>4)&0x0F);
-		outputString[2*i+1] = bzrtp_byteToChar(inputBytes[i]&0x0F);
-	}
-}
-
-/**
- * @brief	convert an hexa char [0-9a-fA-F] into the corresponding unsigned integer value
- * Any invalid char will be converted to zero without any warning
- *
- * @param[in]	inputChar	a char which shall be in range [0-9a-fA-F]
- *
- * @return		the unsigned integer value in range [0-15]
- */
-uint8_t bzrtp_charToByte(uint8_t inputChar) {
-	/* 0-9 */
-	if (inputChar>0x29 && inputChar<0x3A) {
-		return inputChar - 0x30;
-	}
-
-	/* a-f */
-	if (inputChar>0x60 && inputChar<0x67) {
-		return inputChar - 0x57; /* 0x57 = 0x61(a) + 0x0A*/
-	}
-
-	/* A-F */
-	if (inputChar>0x40 && inputChar<0x47) {
-		return inputChar - 0x37; /* 0x37 = 0x41(a) + 0x0A*/
-	}
-
-	/* shall never arrive here, string is not Hex*/
-	return 0;
-
-}
-
-/**
- * @brief	convert a byte which value is in range [0-15] into an hexa char [0-9a-fA-F]
- *
- * @param[in]	inputByte	an integer which shall be in range [0-15]
- *
- * @return		the hexa char [0-9a-f] corresponding to the input
- */
-uint8_t bzrtp_byteToChar(uint8_t inputByte) {
-	inputByte &=0x0F; /* restrict the input value to range [0-15] */
-	/* 0-9 */
-	if(inputByte<0x0A) {
-		return inputByte+0x30;
-	}
-	/* a-f */
-	return inputByte + 0x57;
-}
 
 #else /* NOT HAVE_LIBXML2 */
 int bzrtp_getSelfZID(bzrtpContext_t *context, uint8_t selfZID[12]) {
@@ -429,7 +352,7 @@ int bzrtp_getSelfZID(bzrtpContext_t *context, uint8_t selfZID[12]) {
 		return ZRTP_ZIDCACHE_INVALID_CONTEXT; 
 	}
 	/* we are running cacheless, return a random number */
-	bzrtpCrypto_getRandom(context->RNGContext, selfZID, 12);
+	bctbx_rng_get(context->RNGContext, selfZID, 12);
 	return 0; 
 }
 int bzrtp_getPeerAssociatedSecretsHash(bzrtpContext_t *context, uint8_t peerZID[12]) {
